/************************************************************************************************
** File:        main.c
** Description: Implementation of sample project to blink LEDs
** Version:		1.0
**
*
** Copyright 2015 Zilog Inc. ALL RIGHTS RESERVED.
*
*************************************************************************************************
* The source code in this file was written by an authorized Zilog employee or a licensed 
* consultant. The source code has been verified to the fullest extent possible. 
*
* Permission to use this code is granted on a royalty-free basis. However, users are cautioned to
* authenticate the code contained herein. 
* 
* ZILOG DOES NOT GUARANTEE THE VERACITY OF THIS SOFTWARE; ANY SOFTWARE CONTAINED HEREIN IS
* PROVIDED "AS IS." NO WARRANTIES ARE GIVEN, WHETHER EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
* IMPLIED WARRANTIES OF FITNESS FOR PARTICULAR PURPOSE OR MERCHANTABILITY. IN NO EVENT WILL ZILOG
* BE LIABLE FOR ANY SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OR ANY LIABILITY IN TORT,
* NEGLIGENCE, OR OTHER LIABILITY INCURRED AS A RESULT OF THE USE OF THE SOFTWARE, EVEN IF ZILOG 
* HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. ZILOG ALSO DOES NOT WARRANT THAT THE USE 
* OF THE SOFTWARE, OR OF ANY INFORMATION CONTAINED THEREIN WILL NOT INFRINGE ANY PATENT, 
* COPYRIGHT, OR TRADEMARK OF ANY THIRD PERSON OR ENTITY.

* THE SOFTWARE IS NOT FAULT-TOLERANT AND IS NOT DESIGNED, MANUFACTURED OR INTENDED FOR USE IN 
* CONJUNCTION WITH ON-LINE CONTROL EQUIPMENT, IN HAZARDOUS ENVIRONMENTS, IN APPLICATIONS 
* REQUIRING FAIL-SAFE PERFORMANCE, OR WHERE THE FAILURE OF THE SOFTWARE COULD LEAD DIRECTLY TO 
* DEATH, PERSONAL INJURY OR SEVERE PHYSICAL OR ENVIRONMENTAL DAMAGE (ALL OF THE FOREGOING, 
* "HIGH RISK ACTIVITIES"). ZILOG SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY TO HIGH 
* RISK ACTIVITIES.
*
************************************************************************************************/


#include "Z32F1281.h"
#include "timer.h"


#define EXT_OSC			__bita(SCU->CMR,0)
#define PLL_LOCKED  __bita(SCU->PLLCON,12)

/*********************************************************************
 *	Init_SysClock
 *	
 *	Initializes System Clock using external Oscillator with PLL to 
 *	run at 80Mhz. 
 *	
 *	Requires:
 *		Nothing
 *	Returns: 
 *		Nothing
 **********************************************************************/
 
void Init_SysClock(void)
{
    
	uint32_t x;
	SCU->PER1 |= (1 << SCU_PER1_GPIOC_Pos);		//Enable PC port
	SCU->PCER1 |= (1 << SCU_PCER1_GPIOC_Pos);		//Enable PC Clock
	
    
	PORTEN->EN = 0x15;				//Unlock PCU
	PORTEN->EN = 0x51;
	
	x = PCC->MR;
	x &= ~(PCC_MR_P12_Msk | PCC_MR_P13_Msk);  	//Clear PC12/PC13 Alt Function
	x |= (1 << PCC_CR_P12_Pos) | (1 << PCC_CR_P13_Pos);		//Set PC12/PC13 for Alt Function 1 XTAL
	PCC->MR = x;
	
	PORTEN->EN = 0x00;				//Lock PCU	

	SCU->CMR |= (1 << SCU_CMR_MCLKMNT_Pos) | 
                (1 << SCU_CMR_EOSCIE_Pos)  |
                (1 << SCU_CMR_EOSCMNT_Pos);		//Enable External XTAL monitoring
	
    SCU->CSCR = (2 << SCU_CSCR_EOSCON_Pos)    | 
                (2 << SCU_CSCR_RINGOSCCON_Pos) |
                (2 << SCU_CSCR_IOSCCON_Pos);               //Make sure All Clocks are enabled


	while (! EXT_OSC)				//Wait for external XTAL to start oscillating
	{
		__NOP();
	}
		
	for (x=0;x<1000;++x) ;		//Let it stablize

	SCU->SCCR = (1 << SCU_SCCR_FINSEL_Pos) | (2 << SCU_SCCR_MCLKSEL_Pos);     //Select External oscillator, Bypass PLL 
	SCU->PLLCON = (1 << SCU_PLLCON_PLLRSTB_Pos) |
                  (1 << SCU_PLLCON_PLLEN_Pos)   |
                  (2 << SCU_PLLCON_FBCTRL_Pos)  |
                  (0 << SCU_PLLCON_POSTDIV_Pos);
   	
	while(!(PLL_LOCKED))	//wait until PLL is locked (stable).
	{
		__NOP();
	}
	
	SCU->PLLCON |= (1 << SCU_PLLCON_BYPASS_Pos);         
    
    SCU->SCCR |= (1 << SCU_SCCR_MCLKSEL_Pos);    //Output clock is PLL, Ext XTAL clock is used for FIN.
}	

/*********************************************************************
 *	ClockOut
 *	
 *	Initializes Clock Out for montoring Clock system.
 *	
 *	Requires:
 *		Nothing
 *	Returns: 
 *		Nothing
 **********************************************************************/
void ClockOut(void)
{
	PORTEN->EN = 0x15;				//Unlock PCU
	PORTEN->EN = 0x51;
	
	PCC->MR |= (1 << PCC_MR_P9_Pos);  	//Alt Function PC9
	PCC->CR &= ~PCC_CR_P9_Msk;       	//PC9 output
	
	PORTEN->EN = 0x00;				//Lock PCU
	
	SCU->COR = (1 <<SCU_COR_CLKOEN_Pos) | (1 << SCU_COR_CLKODIV_Pos);	//Enable output and divide by 4
}

/*********************************************************************
 *	Init_LEDs
 *	
 *	Initializes the GPIO pins that control the LEDs (PD8-PD9)
 *	and keep LEDs off until the first timer IRQ fires
 *	Requires:
 *		Nothing
 *	Returns: 
 *		Nothing
 **********************************************************************/
void Init_LEDs(void)
{
	SCU->PER1 |= (1 << SCU_PER1_GPIOD_Pos);			//Enable PD port
	SCU->PCER1 |= (1 << SCU_PCER1_GPIOD_Pos);		//Enable PD port clock
	
	PD->BCR = 0x0300;							//Make sure PD8, PD9 are off
	
	PORTEN->EN = 0x15;						//Unlock PCU
	PORTEN->EN = 0x51;
	
	PCD->MR &= ~(PCD_MR_P8_Msk | PCD_MR_P9_Msk);			//GPIO mode
	PCD->CR &= ~(PCD_CR_P8_Msk | PCD_CR_P9_Msk);         //Output
	
	PORTEN->EN = 0;								// Lock PCU
}

/*********************************************************************
 *	main()
 *	
 *	Application Entry. Initializes periphreals and then runs an 
 *	infinite loop, letting interrupts turn LEDs on and off
 *	
 *	Requires:
 *		Nothing
 *	Returns: 
 *		Nothing
 **********************************************************************/
int main(void)
{
	Init_SysClock();
	ClockOut();
	
	Init_LEDs();
	Init_Timer();
	
	while (1)
	{
		__NOP();			//Do nothing, let the interrupt handle it
	}
}

